package com.fanruan.api.data.open;

import com.fr.data.AbstractParameterTableData;
import com.fr.general.data.DataModel;
import com.fr.script.Calculator;
import com.fr.stable.ParameterProvider;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-09-04
 * 插件的数据集接口
 */
public abstract class BaseTableData extends AbstractParameterTableData {

    /**
     * 实现这个方法来返回一个数据集
     *
     * @param calculator 算子
     * @return 数据集二维表对象
     */
    @Override
    public abstract DataModel createDataModel(Calculator calculator);

    /**
     * 返回获取数据的执行对象
     * 系统取数时，调用此方法来返回一个获取数据的执行对象
     * 注意！ 当数据集需要根据不同参数来多次取数时，此方法在一个计算过程中会被多次调用。
     *
     * @param calculator 算子
     * @param rowCount   要获取数据的行数
     * @return 数据集二维表对象
     */
    @Override
    public DataModel createDataModel(Calculator calculator, int rowCount) {
        return createDataModel(calculator);
    }

    /**
     * 返回获取数据的执行对象
     * 系统取数时，调用此方法来返回一个获取数据的执行对象
     * 注意！ 当数据集需要根据不同参数来多次取数时，此方法在一个计算过程中会被多次调用。
     *
     * @param calculator 算子
     * @param name       数据集的名字
     * @return 数据集二维表对象
     */
    @Override
    public DataModel createDataModel(Calculator calculator, String name) {
        return createDataModel(calculator);
    }

    /**
     * 获取数据集中定义的参数
     *
     * @param calculator 算子
     * @return 参数数组
     */
    @Override
    public ParameterProvider[] getParameters(Calculator calculator) {
        return super.getParameters(calculator);
    }
}
