package com.fanruan.api.design.work.form.creator;

import com.fanruan.api.log.LogKit;
import com.fr.design.designer.creator.CRPropertyDescriptor;

import java.beans.IntrospectionException;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-09-10
 */
public class Attribute {

    private CRPropertyDescriptor descriptor;

    public static Attribute newAttribute(String name, Class<?> beanClass) {
        return new Attribute(name, beanClass);
    }

    public static Attribute newAttribute(String name, Class<?> beanClass, String readMethod, String writeMethod) {
        return new Attribute(name, beanClass, readMethod, writeMethod);
    }

    private Attribute(String name, Class<?> beanClass) {
        try {
            descriptor = new CRPropertyDescriptor(name, beanClass);
        } catch (IntrospectionException e) {
            LogKit.error(e.getMessage(), e);
        }
    }

    private Attribute(String name, Class<?> beanClass, String readMethod, String writeMethod) {
        try {
            descriptor = new CRPropertyDescriptor(name, beanClass, readMethod, writeMethod);
        } catch (IntrospectionException e) {
            LogKit.error(e.getMessage(), e);
        }
    }

    public Attribute keyValue(String name, Object value) {
        if (descriptor != null) {
            descriptor.putKeyValue(name, value);
        }
        return this;
    }

    public Attribute i18n(String i18nName) {
        if (descriptor != null) {
            descriptor.setI18NName(i18nName);
        }
        return this;
    }

    public Attribute editorClass(Class<?> clazz) {
        if (descriptor != null) {
            descriptor.setEditorClass(clazz);
        }
        return this;
    }

    public Attribute rendererClass(Class<?> clazz) {
        if (descriptor != null) {
            descriptor.setRendererClass(clazz);
        }
        return this;
    }

    /**
     * 这个属于内部调用的方法，外部不应该调用
     */
    protected CRPropertyDescriptor toPropertyDescriptor() {
        return descriptor;
    }
}
