package com.fanruan.api.util;

import com.fr.stable.os.AbstractOperatingSystem;
import com.fr.stable.os.Arch;
import com.fr.stable.os.OperatingSystem;

/**
 * 操作系统判断类，判断操作系统种类
 * @author Kalven
 * @version 10.0
 * Created by Kalven on 2019/8/31
 */
public class OperatingKit {

    /**
     * 得到当前操作系统信息
     */
    private static final AbstractOperatingSystem myOS = com.fr.stable.os.OperatingSystem.getOperatingSystem();

    private OperatingKit() {
    }

    /**
     * 获取当前机器的操作系统类型
     * @return 操作系统类型
     */
    public static OperatingSystem getOperatingSystem() {
        return myOS.getType();
    }

    /**
     * 获取当前机器的CPU架构
     * @return CPU架构
     */
    public static Arch getArch() {
        return myOS.getArch();
    }


    /**
     * 判断系统是否是Windows
     * @return 判断结果
     */
    public static boolean isWindows() {
        return myOS.getType() == com.fr.stable.os.OperatingSystem.WINDOWS;
    }

    /**
     * 判断系统是否是Unix
     * @return 判断结果
     */
    public static boolean isUnix() {
        return myOS.isUnix();
    }

    /**
     * 判断系统是否是MacOS
     * @return 判断结果
     */
    public static boolean isMacOS() {
        return myOS.getType() == com.fr.stable.os.OperatingSystem.MACOS;
    }

}
